@@ -1,6 +1,6 @@
 package Net::OSCAR::Callbacks;
 
-$VERSION = 0.01;
+$VERSION = 0.05;
 
 use strict;
 use vars qw($VERSION);
@@ -224,9 +224,9 @@ sub process_snac($$) {
 		$connection->snac_put(family => 0x01, subtype => 0x06);
 	} elsif($family == 0x4 and $subtype == 0x7) {
 		$connection->debug_print("Got incoming IM.");
-		my($from, $msg, $away, $chat) = $session->im_parse($data);
+		my($from, $msg, $away, $chat, $chaturl) = $session->im_parse($data);
 		if($chat) {
-			$session->callback_chat_invite($from, $msg, $chat);
+			$session->callback_chat_invite($from, $msg, $chat, $chaturl);
 		} else {
 			$session->callback_im_in($from, $msg, $away);
 		}
@@ -283,8 +283,6 @@ sub process_snac($$) {
 
 		#$session->debug_print("Requesting chatnav.");
 		#$session->svcreq(CONNTYPE_CHATNAV);
-
-		$session->callback_signon_done();
 	} elsif($family == 0x13 and $subtype == 0x6) {
 		$connection->debug_print("Got buddylist 0x0006.");
 		my $tlvlen = 0;
@@ -391,6 +389,8 @@ sub process_snac($$) {
 				buddyid => $buddy->{buddyid}
 			};
 		}
+
+		$session->callback_signon_done();
 	} elsif($family == 0x13 and $subtype == 0x0E) {
 		$connection->debug_print("Got blmod ack.");
 		$session->modgroups();
@@ -494,15 +494,15 @@ sub process_snac($$) {
 		my($subreq) = unpack("n", $tlv->{0x3}) if $tlv->{0x3};
 		$subreq ||= 0;
 		if($reqtype == 2) {
-			$reqdesc = "password change";
+			$reqdesc = ADMIN_TYPE_PASSWORD_CHANGE;
 		} elsif($reqtype == 3) {
 			if($subreq == 0x11) {
-				$reqdesc = "email change";
+				$reqdesc = ADMIN_TYPE_EMAIL_CHANGE;
 			} else {
-				$reqdesc = "screenname format";
+				$reqdesc = ADMIN_TYPE_SCREENNAME_FORMAT;
 			}
 		} elsif($reqtype == 0x1E) {
-			$reqdesc = "account confirm";
+			$reqdesc = ADMIN_TYPE_ACCOUNT_CONFIRM;
 		}
 		$reqdesc ||= sprintf "unknown admin reply type 0x%04X/0x%04X", $reqtype, $subreq;
 
@@ -514,15 +514,15 @@ sub process_snac($$) {
 			} else {
 				my($result) = unpack("n", $tlv->{0x08});
 				if($result == 2) {
-					$errdesc = "Incorrect password.";
+					$errdesc = ADMIN_ERROR_BADPASS;
 				} elsif($result == 6) {
-					$errdesc = "Invalid input.";
+					$errdesc = ADMIN_ERROR_BADINPUT;
 				} elsif($result == 0xB or $result == 0xC) {
-					$errdesc = "The screenname/email/password is too long or too short.";
+					$errdesc = ADMIN_ERROR_BADLENGTH;
 				} elsif($result == 0x13) {
-					$errdesc = "Your request could not be processed.  Wait a few minutes and try again.";
+					$errdesc = ADMIN_ERROR_TRYLATER;
 				} elsif($result == 0x1D) {
-					$errdesc = "You already have a pending email change request.";
+					$errdesc = ADMIN_ERROR_REQPENDING;
 				} else {
 					$errdesc = sprintf "Unknown error 0x%04X.", $result;
 				}
